#!/usr/bin/env python
"""
Main.py
March 26, 2011

The main recent-notifications-unity application.
"""

import logging
import os

from gi.repository import GObject, Gtk, Unity

import Timestamp

from About import show_about
from MessageBox import MessageBox
from Notification import Notification

logger = logging.getLogger("Main")

SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))

class MainWindow(Gtk.Window):
  def __init__(self):
    GObject.GObject.__init__(self)

    self._default_size = 400

    self.set_title("Recent Notifications")
    self.set_default_size(self._default_size, self._default_size)
    self.connect("destroy", self.on_destroy)
    self.connect("focus-in-event", self.on_focus_in)
    self.connect("focus-out-event", self.on_focus_out)
    self.connect("size-allocate", self.on_size_allocate)

    self._message_count = 0

    self._uimanager = Gtk.UIManager()
    self.add_accel_group(self._uimanager.get_accel_group())

    self._actiongroup = Gtk.ActionGroup("MenuBar")
    self._actiongroup.add_actions([
      ("File", None, "_File", None, None, None),
      ("Clear", Gtk.STOCK_CLEAR, "_Clear Messages", None, None, self.on_clear),
      ("Quit", Gtk.STOCK_QUIT, None, None, None, self.on_destroy),
      ("Help", None, "_Help", None, None, None),
      ("About", Gtk.STOCK_ABOUT, None, None, None, self.on_show_about)
    ])
    self._uimanager.insert_action_group(self._actiongroup)

    self._uimanager.add_ui_from_string("""
      <ui>
        <menubar name="MenuBar">
          <menu action="File">
            <menuitem action="Clear"/>
            <separator/>
            <menuitem action="Quit"/>
          </menu>
          <menu action="Help">
            <menuitem action="About"/>
          </menu>
        </menubar>
      </ui>
    """)

    self._vbox = Gtk.VBox()
    self.add(self._vbox)

    self._menubar = self._uimanager.get_widget("/MenuBar")
    self._vbox.pack_start(self._menubar, False, False, 0)

    self._message_box = MessageBox()
    self._vbox.pack_start(self._message_box, True, True, 0)
    
    self._launcher = Unity.LauncherEntry.get_for_desktop_id("rn-unity.desktop")

    self._notify = Notification()
    self._notify.connect("message-received", self.on_message_received)

  def zero_count(self):
    """Zero the message count"""
    self._message_count = 0
    self.update_count()

  def increment_count(self):
    """Increments the message count"""
    self._message_count += 1
    self.update_count()

  def update_count(self):
    """Updates the message count display"""
    self._launcher.set_property("count", self._message_count)
    if self._message_count > 0:
      self._launcher.set_property("count_visible", True)
    else:
      self._launcher.set_property("count_visible", False)

  def on_clear(self, *args):
    """Clear the messages"""
    self._message_box.clear_messages()

  def on_destroy(self, *args):
    """Quit the application"""
    Gtk.main_quit()

  def on_focus_in(self, widget, event):
    """Window receives focus"""
    pass

  def on_focus_out(self, widget, event):
    """Window receives focus"""
    self.zero_count()
    self._message_box.mark_as_read()

  def on_message_received(self, monitor, message):
    """Received a notification"""
    self.increment_count()
    self._message_box.add_message(message)

  def on_size_allocate(self, widget, size):
    """Resize event"""
    pass

  def on_show_about(self, *args):
    """Show the about dialog"""
    show_about()

def main():
  log_path = os.path.join(SCRIPT_PATH, "rn-unity.log")
  logging.basicConfig(filename=log_path, level=logging.DEBUG)

  icon_path = os.path.join(SCRIPT_PATH, "..", "icons")
  theme = Gtk.IconTheme.get_default()
  theme.append_search_path(icon_path)

  window = MainWindow()
  window.show_all()
  Gtk.main()

if __name__ == "__main__":
  main()
